/**
 *
 * xpt -- An X Periodic Table
 *
 * table.c - Create the periodic table, and handle data requests
 *
 * Written bu Joel P. Lord 03/05/93
 *
 * Modified (elements 104-110 added etc.) by Kai Nordlund Dec 1994
 * Modified (elements 111- added ) by Kai Nordlund 
 *
 *    This software is available for free distribution,
 * under the condition that this not be removed from the
 * source code.
 *
 * Note: if the compiler complains about XtPointer, try placing a define
 * #define XtPointer void * at the beginning of this file.
**/

#include "xpt.h"
#include <X11/Xaw/AsciiText.h>

void init_table(int structcolors)
{
  Widget elements[126];
  Widget quit_button,genhelp_button;
  int n = 0, i = 0, j = 0, k = 0, o = 0, l, horiz_offset = 5;
  int ii;
  Arg args[30];
  char title[5];
  char str[4];
  void element_choice(),quit(),genhelp();
  static char *table[] = {
"H                                                  He ",
"Li Be                               B  C  N  O  F  Ne ",
"Na Mg                               Al Si P  S  Cl Ar ",
"K  Ca Sc Ti V  Cr Mn Fe Co Ni Cu Zn Ga Ge As Se Br Kr ",
"Rb Sr Y  Zr Nb Mo Tc Ru Rh Pd Ag Cd In Sn Sb Te I  Xe ",
"Cs Ba La Hf Ta W  Re Os Ir Pt Au Hg Tl Pb Bi Po At Rn ",
"Fr Ra Ac Rf Db Sg Bh Hs Mt Ds 111112113114115116   118",
"                                                      ",
"         Ce Pr Nd Pm Sm Eu Gd Tb Dy Ho Er Tm Yb Lu    ",
"         Th Pa U  Np Pu Am Cm Bk Cf Es Fm Md No Lr    "
};
  unsigned long color;

  /* KN To find the possible arguments here, look into 
     /usr/X11R6/include/X11/Xaw/Command.h
  */

  n = 0;
  XtSetArg(args[n], XtNfromHoriz, NULL);  // 0
  n++;
  XtSetArg(args[n], XtNfromVert, NULL);
  n++;
  XtSetArg(args[n], XtNhorizDistance, 5);
  n++;
  XtSetArg(args[n], XtNvertDistance, 5);
  n++;
  XtSetArg(args[n], XtNwidth, 25);
  n++;
  XtSetArg(args[n], XtNheight, 32);
  n++;
  mbgpix = GetColors("lightgrey", default_cmap, BP);
  XtSetArg(args[n], XtNbackground, slate_grey);
  n++;
  XtSetArg(args[n], XtNforeground, mfgpix);
  n++;
  XtSetArg(args[n], XtNfont, mfontstruct);
  n++;
  /* KN to adjust centering of numerical element names */
  XtSetArg(args[n], XtNinternalWidth, 2); // 9
  n++;
  XtSetArg(args[n], XtNborderWidth, 1); // 10
  n++;
  title[3] = ' ';
  title[4] = 0;
  for (j = 0; j < 10; j++)
    {
      for(i = 0; i < 54; i += 3)
	{
	  ii=i/3;
	  title[0] = table[j][i];
	  title[1] = table[j][i+1];
	  if (title[1]==' ') title[1]=0;
	  /* Elements 110 - */
	  title[2] = table[j][i+2];
	  if (title[2]==' ') title[2]=0; 
	  else title[3]=0;
	  if (title[0] != ' ')
	    {
              /* Select element color according to type */
              color=slate_grey;
	      /* printf("ii %d",ii); fflush(stdout); */
	      if (!structcolors) {
		if (ii==17) color=noble;
		else if (ii-j>11 || k==4) color=nonmetal;
		else if ((ii-j>10 || k==31)&&j<6) color=semimetal;
		else if ((ii==2 && j==5) || j==8) color=lantanide;
		else if ((ii==2 && j==6) || j==9) color=actinide;
		else if (ii==0) color=alkali;
		else if (ii==1) color=earthalkali;
		else color=metal; 
	      }
	      else {
		if (strncmp(p_table[k].crystal,"BCC",3)==0) 
		  color=alkali;
		else if (strncmp(p_table[k].crystal,"HEX",3)==0) 
		  color=earthalkali;
		else if (strncmp(p_table[k].crystal,"FCC",3)==0) 
		  color=metal;
		else if (strncmp(p_table[k].crystal,"DIA",3)==0) 
		  color=semimetal;
		else if (strncmp(p_table[k].crystal,"   ",3)==0) 
		  color=mbgpix;
		else color=actinide;
	      }
              XtSetArg(args[6], XtNbackground, color);
	      elements[k] = XtCreateWidget(title, commandWidgetClass,
					   MainW, args, n);
	      XtAddCallback(elements[k], XtNcallback, element_choice, k);
	      XtRealizeWidget(elements[k]);
	      XtManageChild(elements[k]);
	      /* sprintf(str,"%d %d\0",k,p_table[k].atomic_number);
		 XDrawString(p_disp, XtWindow(elements[k]), SmallGC, 1, 
		 8,str,strlen(str)); */
	      k++;
	      horiz_offset += 26;
	      XtSetArg(args[2], XtNhorizDistance, horiz_offset);
	    }
	  else
	    {
	      horiz_offset += 26;
	      XtSetArg(args[2], XtNhorizDistance, horiz_offset);
	    }
	}
      /* printf("i %d j %d k %d",i,j,k); fflush(stdout); */
      horiz_offset = 5;
      if (j>6) horiz_offset=10;
      XtSetArg(args[0], XtNfromHoriz, NULL);
      XtSetArg(args[1], XtNfromVert, elements[k-1]);
      XtSetArg(args[2], XtNhorizDistance, 5);
      if (j==7) XtSetArg(args[3], XtNvertDistance, 4);
      else XtSetArg(args[3], XtNvertDistance,-1);/*KN to get overlap */
    }

  XtSetArg(args[0], XtNfromHoriz, NULL);
  XtSetArg(args[1], XtNfromVert, NULL);
  XtSetArg(args[2], XtNhorizDistance, 150);
  XtSetArg(args[3], XtNvertDistance, 5);
  XtSetArg(args[4], XtNwidth, 49);
  XtSetArg(args[5], XtNheight, 32);
  XtSetArg(args[6], XtNbackground, bcol);
  XtSetArg(args[7], XtNforeground, white);
  XtSetArg(args[8], XtNfont, mfontstruct);
  XtSetArg(args[9], XtNinternalWidth, 0);
  XtSetArg(args[10], XtNborderWidth, 0); 

  quit_button = XtCreateManagedWidget("Quit", commandWidgetClass, MainW,
  				      args, 11);
  XtRealizeWidget(quit_button);
  XtAddCallback(quit_button, XtNcallback, quit, 0);

  XtSetArg(args[2], XtNhorizDistance, 150+55);

  genhelp_button = XtCreateManagedWidget("Info", commandWidgetClass, MainW,
				      args, 11);
  XtRealizeWidget(genhelp_button);
  XtAddCallback(genhelp_button, XtNcallback, genhelp, 0);


}

static int destroy_genhelp;
static int destroy_info=1;

void genhelp(w, client_data, call_data)
Widget w;
XtPointer client_data;
XtPointer call_data;
{
  Widget genhelp_info,ok_button;
  void genhelpok();

  destroy_genhelp=0;

   genhelp_info = XtVaCreateManagedWidget("General Help",formWidgetClass, MainW,
				       XtNfromHoriz, NULL,
				       XtNfromVert, NULL,
				       XtNwidth, 450,
				       XtNheight, 220,
				       XtNvertDistance, 50,
				       XtNhorizDistance, 15,
				       XtNbackground, mbgpix,
				       XtNforeground, mfgpix,
				       XtNfont, mfontstruct,
				       XtNborderColor, bcol, NULL);
  display_genhelp(genhelp_info);

  ok_button = XtVaCreateManagedWidget("Done", commandWidgetClass, genhelp_info,
				    XtNfromHoriz, NULL,
				    XtNfromVert, NULL,
				    XtNwidth, 350,
				    XtNheight, 20,
				    XtNvertDistance, 220-30,
				    XtNhorizDistance, 50,
				    XtNbackground, bcol,
				    XtNforeground, mfgpix,
				    XtNfont, mfontstruct,
				    XtNborderColor, mbgpix, NULL);

  XtAddCallback(ok_button, XtNcallback, genhelpok, 0);

  while (!destroy_genhelp && !done)
    {
      XtNextEvent(&theEvent);
      XtDispatchEvent(&theEvent);
      if (theEvent.type == Expose)
	display_genhelp(genhelp_info);
    }
  XtDestroyWidget(genhelp_info);

}

void genhelpok(w, client_data, call_data)
Widget w;
XtPointer client_data;
XtPointer call_data;
{
  destroy_genhelp = 1;
}

display_genhelp(w)
Widget w;
{
  static char *s[]={
"Xpt has been written by Joel P. Lord and modified by Kai Nordlund.",
"The numerical data on the elements comes from several sources.",
" ",
"The info on elements 104-109 is from http://www.cchem.berkeley.edu/Table/",
"The names of these elements are the new (Mar -97) IUPAC compromise",
"The very recent (1994 or newer) info on Z>109 is from http://www.gsi.de/",
"The data on elements 114-118 is from various web resources.",
" ",
"The crystal data and Debye temperatures are from Aschcroft-Mermin.", 
"The lattice c/a ratio is actually the angle alpha for an RHL lattice and",
"the b/a ratio in ORC. A minus in the Debye temperature means a low",
"temperature determination.",
" ",
"Please report any errors to kai.nordlund@helsinki.fi."
};
  int i,j;

  j=3;
  for (i=0;i<14;i++) {
     if (s[i][0]==' ') j+=7;
     else j+=14;
     XDrawString(p_disp, XtWindow(w), theGC, 10, j, s[i],strlen(s[i]));
  }
}

static int xpos;
Widget ok_button, help_button;


void element_choice(w, element, event)
Widget w;
int element;
XEvent *event;
{
  Widget element_info;
  Arg args[20];
  char *data_buf;
  void ok(), help();
  int k;

/*
   Element numbers are as in the table in xpt.h:

   0 is H, 1 is He, 3 - 10 is Li - Ne, 11 - 18 is Na - Ar,
   19 - 35 is K - Kr, 36 - 53 is Rb - Xe, 54 - 71 is Cs - Rn,
   72 - 88 is Fr - 118, 89 - 102 is Ce - Lu, 103 - 116 is Th - Lr  

  (i.e. the series "breaks" at the lanthanides and actinides)

*/

  k=element;
  xpos=260;
  if (k==1 || (k>3&&k<10) || (k>11&&k<18) ||
      (k>26&&k<36) || (k>44&&k<54) ||
      (k>62&&k<72) || (k>80&&k<89) ||
      (k>94&&k<103) || (k>108) ) xpos=20;

  data_buf = (char *) malloc(1000);

  destroy_info = 0;

  element_info = XtVaCreateManagedWidget("Element Info",formWidgetClass, MainW,
				       XtNfromHoriz, NULL,
				       XtNfromVert, NULL,
				       XtNwidth, 200,
				       XtNheight, 250,
				       XtNvertDistance, 60,
				       XtNhorizDistance, xpos,
				       XtNbackground, mbgpix,
				       XtNforeground, mfgpix,
				       XtNfont, mfontstruct,
				       XtNborderColor, bcol, NULL);
  /* printf("element %d %d\n",element,k); */
  display_info(element_info, element);

  ok_button = XtVaCreateManagedWidget("Done", commandWidgetClass, element_info,
				    XtNfromHoriz, NULL,
				    XtNfromVert, NULL,
				    XtNwidth, 40,
				    XtNheight, 32,
				    XtNvertDistance, 210,
				    XtNhorizDistance, 100,
				    XtNbackground, bcol,
				    XtNforeground, mfgpix,
				    XtNfont, mfontstruct,
				    XtNborderColor, mbgpix, NULL);

  XtAddCallback(ok_button, XtNcallback, ok, 0);

  help_button = XtVaCreateManagedWidget("Help", commandWidgetClass,
					element_info,
					XtNfromHoriz, NULL,
					XtNfromVert, NULL,
					XtNwidth, 40,
					XtNheight, 32,
					XtNvertDistance, 210,
					XtNhorizDistance, 60,
					XtNbackground, bcol,
					XtNforeground, mfgpix,
					XtNfont, mfontstruct,
					XtNborderColor, mbgpix, NULL);
  XtAddCallback(help_button, XtNcallback, help, 0);

  while (!destroy_info && !done)
     {
	XtNextEvent(&theEvent);
	XtDispatchEvent(&theEvent);
	if (theEvent.type == Expose) {
	   display_info(element_info, element);
	}
	if (theEvent.type == ButtonPress && 
	    theEvent.xany.window != XtWindow(help_button)) {
	   destroy_info=1;
	}
	XtMoveWidget(element_info,xpos,60);
     }

  XtDestroyWidget(element_info);
  destroy_info=1;
  free(data_buf);
}

void ok(w, client_data, call_data)
Widget w;
XtPointer client_data;
XtPointer call_data;
{
  destroy_info = 1;
}

void quit(w, client_data, call_data)
Widget w;
XtPointer client_data;
XtPointer call_data;
{
  done = 1;
}

static unsigned destroy_help = 1;
static Widget help_window;

void help(w, client_data, call_data)
Widget w;
XtPointer client_data;
XtPointer call_data;
{
   int x;


   if (xpos < 200) x=xpos+200; 
   else x=xpos-200;

  if (!destroy_help)
    {
      XtDestroyWidget(help_window);
      destroy_help = 1;
    }
  else
    {
      destroy_help = 0;

      help_window = XtVaCreateManagedWidget("Element Info",formWidgetClass,
					    MainW,
					    XtNfromHoriz, NULL,
					    XtNfromVert, NULL,
					    XtNwidth, 200,
					    XtNheight, 250,
					    XtNvertDistance, 60,
					    XtNhorizDistance, x,
					    XtNbackground, mbgpix,
					    XtNforeground, mfgpix,
					    XtNfont, mfontstruct,
					    XtNborderColor, bcol, NULL);
      display_help(help_window);
      
      while (!destroy_info && !done && !destroy_help)
	{
	  XtNextEvent(&theEvent);
	  XtDispatchEvent(&theEvent);
	  if(theEvent.type == Expose)
	    display_help(help_window);
	  if (theEvent.type == ButtonPress) {
	     destroy_help=1;
	     if (theEvent.xany.window != XtWindow(help_button))
		 destroy_info=1;
	  }
	}
      XtDestroyWidget(help_window);
      destroy_help = 1;
    }
}

display_help(w)
Widget w;
{
  XDrawString(p_disp, XtWindow(w), theGC, 10, 10 + mfontheight, "Number", 6);
  XDrawString(p_disp, XtWindow(w), BigGC,
	      100 - XTextWidth(BigFont, "Sy", 2) / 2, 110 + mfontheight,
	      "Sy", 2);
  XDrawString(p_disp, XtWindow(w), theGC, 
	      190 - XTextWidth(mfontstruct, "Mass", 4),
	      10 + mfontheight, "Mass", 4);

  XDrawString(p_disp, XtWindow(w), theGC, 10, 45 + mfontheight, "BP", 2);
  XDrawString(p_disp, XtWindow(w), theGC, 10, 80 + mfontheight, "MP", 2);
  XDrawString(p_disp, XtWindow(w), theGC, 10, 115 + mfontheight, "Density", 7);
  XDrawString(p_disp, XtWindow(w), theGC, 10, 150 + mfontheight, "Lattice a", 9);
  XDrawString(p_disp, XtWindow(w), theGC, 10, 185 + mfontheight, "El. Conf./Lifetime", 18);

  XDrawString(p_disp, XtWindow(w), theGC,
	      185 - XTextWidth(mfontstruct, "Atomic", 6),
	      40 + mfontheight, "Atomic", 6);
  XDrawString(p_disp, XtWindow(w), theGC,
	      190 - XTextWidth(mfontstruct, "Radius", 6),
	      40 + 2 * mfontheight, "Radius", 6);

  XDrawString(p_disp, XtWindow(w), theGC,
	      185 - XTextWidth(mfontstruct, "Covalent", 8),
	      75 + mfontheight, "Covalent", 8);
  XDrawString(p_disp, XtWindow(w), theGC,
	      190 - XTextWidth(mfontstruct, "Radius", 6),
	      75 + 2 * mfontheight, "Radius", 6);

  XDrawString(p_disp, XtWindow(w), theGC,
	      185 - XTextWidth(mfontstruct, "Atomic", 6),
	      110 + mfontheight, "Atomic", 6);
  XDrawString(p_disp, XtWindow(w), theGC,
	      190 - XTextWidth(mfontstruct, "Volume", 6),
	      110 + 2 * mfontheight, "Volume", 6);

  XDrawString(p_disp, XtWindow(w), theGC,
	      185 - XTextWidth(mfontstruct, "Lattice", 7),
	      145 + mfontheight, "Lattice", 7);
  XDrawString(p_disp, XtWindow(w), theGC,
	      190 - XTextWidth(mfontstruct, "c/a_ratio", 9),
	      145 + 2 * mfontheight, "c/a ratio", 9);

  XDrawString(p_disp, XtWindow(w), theGC,
	      185 - XTextWidth(mfontstruct, "Debye", 5),
	      180 + mfontheight, "Debye", 5);
  XDrawString(p_disp, XtWindow(w), theGC,
	      190 - XTextWidth(mfontstruct, "Temp.", 5),
	      180 + 2 * mfontheight, "Temp.", 5);

  XDrawString(p_disp, XtWindow(w), theGC,
	      100 - XTextWidth(mfontstruct, "Crystal", 7) / 2,
	      145 + mfontheight, "Crystal", 7);
  XDrawString(p_disp, XtWindow(w), theGC,
	      100 - XTextWidth(mfontstruct, "Type", 4) / 2,
	      145 + 2*mfontheight, "Type", 4);
 
  XDrawString(p_disp, XtWindow(w), theGC,
	      100 - XTextWidth(mfontstruct, "Name", 4) / 2,
	      30 + mfontheight, "Name", 4);
}

display_info(w, element)
Widget w;
unsigned element;
{
  char *temp_buf, *temp2;

  temp_buf = (char *) malloc(100);

/* Left column */

  sprintf(temp_buf, "%d", p_table[element].atomic_number);
  XDrawString(p_disp, XtWindow(w), theGC, 10, 
	      10 + mfontheight, temp_buf,
	      strlen(temp_buf));


  if (p_table[element].boiling_pt > 0.0)
    sprintf(temp_buf, "%.2lf", p_table[element].boiling_pt);
  else
    strcpy(temp_buf, "--");
  XDrawString(p_disp, XtWindow(w), theGC, 
	      10, 45 + mfontheight, temp_buf,
	      strlen(temp_buf));

  if (p_table[element].melting_pt > 0.0)
    sprintf(temp_buf, "%.2lf", p_table[element].melting_pt);
  else
    strcpy(temp_buf, "--");
  XDrawString(p_disp, XtWindow(w), theGC, 
	      10, 80 + mfontheight, temp_buf,
	      strlen(temp_buf));

  if (p_table[element].density > 0.0)
    sprintf(temp_buf, "%.2lf", p_table[element].density);
  else
    strcpy(temp_buf, "--");
  XDrawString(p_disp, XtWindow(w), theGC, 
	      10, 115 + mfontheight, temp_buf,
	      strlen(temp_buf));

  if (p_table[element].a > 0.0)
    sprintf(temp_buf, "%.3lf", p_table[element].a);
  else
    strcpy(temp_buf, "--");
  XDrawString(p_disp, XtWindow(w), theGC,
	      10, 150 + mfontheight, temp_buf, strlen(temp_buf));

  strcpy(temp_buf, p_table[element].elconf);
  temp2 = strchr(temp_buf, ' ');
  *temp2 = 0;
  XDrawString(p_disp, XtWindow(w), theGC,
	      10, 185 + mfontheight, temp_buf, strlen(temp_buf));

/* Right column */

  sprintf(temp_buf, "%1.4lf", p_table[element].atomic_mass);
  if (p_table[element].atomic_mass < 0.0) {
    sprintf(temp_buf, "(%1.0lf)", -p_table[element].atomic_mass);
  }
  XDrawString(p_disp, XtWindow(w), theGC,
	      190 - XTextWidth(mfontstruct, temp_buf, strlen(temp_buf)),
	      10 + mfontheight, temp_buf, strlen(temp_buf));

  if (p_table[element].atomic_radius > 0.0)
    sprintf(temp_buf, "%.2lf", p_table[element].atomic_radius);
  else
    strcpy(temp_buf, "--");
  XDrawString(p_disp, XtWindow(w), theGC,
	      190 - XTextWidth(mfontstruct, temp_buf, strlen(temp_buf)),
	      45 + mfontheight, temp_buf, strlen(temp_buf));

  if (p_table[element].covalent_radius > 0.0)
    sprintf(temp_buf, "%.2lf", p_table[element].covalent_radius);
  else
    strcpy(temp_buf, "--");
  XDrawString(p_disp, XtWindow(w), theGC,
	      190 - XTextWidth(mfontstruct, temp_buf, strlen(temp_buf)),
	      80 + mfontheight, temp_buf, strlen(temp_buf));

  if (p_table[element].atomic_volume > 0.0)
    sprintf(temp_buf, "%.2lf", p_table[element].atomic_volume);
  else
    strcpy(temp_buf, "--");
  XDrawString(p_disp, XtWindow(w), theGC,
	      190 - XTextWidth(mfontstruct, temp_buf, strlen(temp_buf)),
	      115 + mfontheight, temp_buf, strlen(temp_buf));

  if (p_table[element].cpera > 0.0)
    sprintf(temp_buf, "%.3lf", p_table[element].cpera);
  else
    strcpy(temp_buf, "--");
  XDrawString(p_disp, XtWindow(w), theGC,
	      190 - XTextWidth(mfontstruct, temp_buf, strlen(temp_buf)),
	      150 + mfontheight, temp_buf, strlen(temp_buf));

  if (p_table[element].tdebye > 0.0)
    sprintf(temp_buf, "%.2lf", p_table[element].tdebye);
  else
    strcpy(temp_buf, "--");
  XDrawString(p_disp, XtWindow(w), theGC,
	      190 - XTextWidth(mfontstruct, temp_buf, strlen(temp_buf)),
	      185 + mfontheight, temp_buf, strlen(temp_buf));

/* Middle column */

  strcpy(temp_buf, p_table[element].name);
  temp2 = strchr(temp_buf, ' ');
  *temp2 = 0;
  XDrawString(p_disp, XtWindow(w), theGC,
	      100 - XTextWidth(mfontstruct, temp_buf, strlen(temp_buf)) / 2,
	      30 + mfontheight, temp_buf, strlen(temp_buf));

  sprintf(temp_buf, "%s", p_table[element].symbol);
  temp2 = strchr(temp_buf, ' ');
  *temp2 = 0;
  XDrawString(p_disp, XtWindow(w), BigGC,
	      100 - XTextWidth(BigFont, temp_buf, strlen(temp_buf)) / 2,
	      110 + mfontheight, temp_buf, strlen(temp_buf));


  strcpy(temp_buf, p_table[element].crystal);
  temp2 = strchr(temp_buf, ' ');
  *temp2 = 0;
  XDrawString(p_disp, XtWindow(w), theGC,
	      100 - XTextWidth(mfontstruct, temp_buf, strlen(temp_buf)) / 2,
	      150 + mfontheight, temp_buf, strlen(temp_buf));


  free(temp_buf);
}

